#! /usr/bin/perl
# settags.pl
# (C)Copyright 2000, 2001 by Hiroshi Takekawa <sian@big.or.jp>
# Last Modified: Mon Jan 15 02:46:16 2001.

# usage:
# % eval `settags.pl < input_data_file`
# or
# % settags.pl < input_data_file > output_shell_script_file
# then read and edit script, execute it.

use strict;
my $settag = "settag";
my $mv = "mv";

sub read_line {
    while ($_ = <>) {
	s/^\s+//;
	next if /^$/;
	next if /^#/;
	chomp;
	return $_;
    }
    return undef;
}

sub process_album ($) {
    my $album = shift;
    my $sourceformat;
    my $global_artist;
    my $global_year;
    my $delimiter;
    my $count;

    $count = 0;
    while ($_ = read_line) {
	if (/^([^=]+)\s+=\s+(.*)$/) {
	    my $name = $1;
	    my $value = $2;

	    if ($name eq "sourceformat") {
		$sourceformat = $value;
	    } elsif ($name eq "artist") {
		$global_artist = $value;
	    } elsif ($name eq "year") {
		$global_year = $value;
	    } elsif ($name eq "delimiter") {
		$delimiter = $value;
	    } else {
		print "Invalid name ", $name, "\n";
		exit;
	    }
	} elsif (/^}$/) {
	    return;
	} else {
	    my ($dfile, $songname, $local_artist, $local_year, $comment) = split /$delimiter/, $_, 5;

	    if (!$dfile) {
		print "Syntax error: $_\n";
		exit;
	    }
	    $dfile =~ s/%%/$delimiter/;
	    if (!$songname) {
		$songname = $dfile;
		$songname =~ s/^\d+_//;
		$songname =~ s/_/ /g;
	    } else {
		$songname =~ s/%%/$delimiter/;
	    }

	    my $artist = $local_artist ? $local_artist : $global_artist;
	    my $year = $local_year ? $local_year : $global_year;
	    my $sfile = sprintf $sourceformat, ++$count;

	    $artist =~ s/%%/$delimiter/;

	    my $comstring;

	    $songname =~ s/"/\\"/g;

	    if (-f $sfile) {
		$comstring = join("", $mv, " \"", $sfile, "\" \"", $dfile, ".mp3\"");
		$comstring =~ s/;//;
		print $comstring, ";\n";
	    }
	    $sfile = join("", $dfile, ".mp3");
	    $comstring = join("", $settag , " -s \"", $songname, "\" -a \"", $artist, "\" -A \"", $album, "\" -y \"", $year, "\" ");
	    $comstring .= join("", "-c \"", $comment, "\" ") if $comment;
	    $comstring .= join("", "\"", $sfile, "\"");
	    $comstring =~ s/;//;
	    print $comstring, ";\n";
	}
    }
}

while ($_ = read_line) {
    if (/^album\s+(.*)\s+{$/) {
	process_album $1;
    }
}
